# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-23 19:05
from __future__ import absolute_import, unicode_literals

from django.db import migrations, models
import thorn.django.models


class Migration(migrations.Migration):

    dependencies = [
        ('webhooks', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='subscriber',
            name='hmac_digest',
            field=models.CharField(
                choices=[
                    ('SHA512', 'sha512'),
                    ('SHA256', 'sha256'),
                    ('SHA1', 'sha1')],
                default='sha256',
                help_text='Specify HMAC digest type (use sha256 if uncertain)',
                max_length=64,
                verbose_name='HMAC Digest Type'),
        ),
        migrations.AddField(
            model_name='subscriber',
            name='hmac_secret',
            field=models.TextField(
                default=thorn.django.models.random_secret64,
                help_text='Specify HMAC secret for endpoints to verify',
                verbose_name='HMAC Secret'),
        ),
    ]
