"""

    thorn.environment
    =================

    Framework integration.

"""
from __future__ import absolute_import, unicode_literals

__all__ = ['envs']


envs = [
    'thorn.environment.django:DjangoEnv',
]
